@@ -2,10 +2,11 @@
  * Please see file COPYING or http://bitzi.com/publicdomain 
  * for more info.
  *
- * $Id: gui.c,v 1.4 2001/04/06 01:48:13 mayhemchaos Exp $
+ * $Id: gui.c,v 1.5 2001/07/24 00:58:04 mayhemchaos Exp $
  */
 
 #include <windows.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include "bitcollider.h"
 #include "list.h"
@@ -25,10 +26,37 @@ int APIENTRY WinMain(HINSTANCE hInstance,
     int    argIndex = 1;
 
     list = create_list();
-    for(; argIndex < __argc; argIndex++)
-    {
-       add_to_list(list, __argv[argIndex]);
-    }
+	if (__argc > 1 && __argv[argIndex][0] == '@')
+	{
+		FILE *temp;
+		char  line[MAX_PATH];
+
+		temp = fopen(__argv[argIndex] + 1, "r");
+		if (temp == NULL)
+		{
+            MessageBox(NULL, "Cannot read file list from shell extension.", "Bitcollider", MB_OK);
+			return 0;
+		}
+
+		for(;;)
+		{
+            if (fgets(line, MAX_PATH, temp) == NULL)
+			    break;
+
+			line[strlen(line) - 1] = 0;
+			add_to_list(list, strdup(line));
+		}
+
+		fclose(temp);
+
+		// get rid of the temp file
+		unlink(__argv[argIndex] + 1);
+	}
+	else
+        for(; argIndex < __argc; argIndex++)
+		{
+            add_to_list(list, __argv[argIndex]);
+		}
 
 	// Perform application initialization:
 	hWnd = init_gui();
