@@ -2,7 +2,7 @@
  * Please see file COPYING or http://bitzi.com/publicdomain 
  * for more info.
  *
- * $Id: plugin.h,v 1.5 2001/03/19 23:38:29 mayhemchaos Exp $
+ * $Id: plugin.h,v 1.7 2001/06/18 07:27:17 mayhemchaos Exp $
  */
 #ifndef PLUGIN_H
 #define PLUGIN_H
@@ -25,7 +25,7 @@ typedef struct _Attribute
     char *value;
 } Attribute;
 
-typedef void *Context;
+typedef void Context;
 
 typedef struct _PluginMethods
 {
@@ -37,11 +37,11 @@ typedef struct _PluginMethods
    
     Attribute        *(*file_analyze)         (const char *fileName);
 
-    Context           (*mem_analyze_init)     (void);
-    void              (*mem_analyze_update)   (Context context, 
-                                               const unsigned char *,
-                                               unsigned bufLen);
-    Attribute        *(*mem_analyze_final)    (Context context);
+    Context          *(*mem_analyze_init)     (void);
+    void              (*mem_analyze_update)   (Context             *context, 
+                                               const unsigned char *buf,
+                                               unsigned             bufLen);
+    Attribute        *(*mem_analyze_final)    (Context *context);
 
     void              (*free_attributes)      (Attribute *attrList);
 
