@@ -2,7 +2,7 @@
  * Please see file COPYING or http://bitzi.com/publicdomain 
  * for more info.
  *
- * $Id: tiger.h,v 1.2 2001/04/03 23:53:49 mayhemchaos Exp $
+ * $Id: tiger.h,v 1.5 2002/07/06 09:38:02 gojomo Exp $
  */
 #ifndef TIGER_H
 #define TIGER_H
@@ -20,9 +20,14 @@ typedef unsigned long int tword;
 typedef unsigned long long int word64;
 #endif
 
-typedef unsigned long word32;
-typedef unsigned char byte;
+typedef unsigned long  word32;
+typedef unsigned short word16;
+
+#ifndef __BYTE__
+#define __BYTE__
+typedef unsigned char  byte;
+#endif
 
 void tiger(word64 *str, word64 length, word64 *res);
-
+ 
 #endif
