@@ -2,47 +2,46 @@
  * Please see file COPYING or http://bitzi.com/publicdomain 
  * for more info.
  *
- * $Id: main.c,v 1.22 2001/05/18 01:00:54 mayhemchaos Exp $
+ * $Id: main.c,v 1.41 2002/07/06 09:30:15 gojomo Exp $
  */
 /*------------------------------------------------------------------------- */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 
 #include "bc_version.h"
 #include "bitcollider.h"
-#include "tigertree.h"
-#include "sha1.h"
+#include "cache.h"
+#include "bitprint.h"
+#include "md5.h"
+#include "ed2k_md4.h"
+#include "ftuuhash.h"
 #include "mp3.h"
 #include "id3.h"
 #include "plugin_man.h"
+#include "tiger.h"
+#include <sys/stat.h>
 #ifndef _WIN32
   #include "browser.h"
+  #include <errno.h>
+  #include <unistd.h>
 #else
   #include <windows.h>
   #include <shellapi.h>
 #endif
 
 /*------------------------------------------------------------------------- */
-b_bool hashCheckPassed = false;
 
-/*------------------------------------------------------------------------- */
-
-#define BUFFER_LEN           4096
-#define FIRST_N_HEX          20
-#define FIRST_N_HEX_SIZE     (FIRST_N_HEX * sizeof(char) * 2)
-#define MAX_ATTR_STRING_LEN  1024
-#define DEFAULT_NUM_ATTRS    16
-#define GROW_NUM_ATTRS       16
-#define SHA_HEXSIZE          ((SHA_DIGESTSIZE * 2 * sizeof(char)) + 1)
-#define TIGER_HEXSIZE        ((TIGERSIZE * 2 * sizeof(char)) + 1)
-#define ONEK_SIZE            1025
-#define EMPTY_SHA            "DA39A3EE5E6B4B0D3255BFEF95601890AFD80709"
-#define ONE_SHA              "356A192B7913B04C54574D18C28D46E6395428AB"
-#define ONEK_SHA             "1009DE2EF6183BD6D8C08F1ED8DF4847E71F7ACB"
-#define EMPTY_TIGER          "3293AC630C13F0245F92BBB1766E16167A4E58492DDE73F3"
-#define ONE_TIGER            "1D573194A056EB3200F9D302900C843C3D41AB4ED06C03DF"
-#define ONEK_TIGER           "D1593128F306DD1C2831459205F14CCB843E675F021F2EA2"
+#define BUFFER_LEN              4096
+#define FIRST_N_HEX             20
+#define FIRST_N_HEX_SIZE        (FIRST_N_HEX * sizeof(char) * 2)
+#define MAX_ATTR_STRING_LEN     1024
+#define DEFAULT_NUM_ATTRS       16
+#define GROW_NUM_ATTRS          16
+#define MD5_SANITY_CHECK_FAILED "The MD5 hash function compiled into the bitcollider is faulty."
+#define MD5_SANITY_CHECK_EMPTY  "2QOYZWMPACZAJ2MABGMOZ6CCPY"
+#define MD5_SANITY_CHECK_01234  "IEAMJVCNVELXER7EJJP4CVDHPA"
 
 #ifdef _WIN32
 #define DIR_SEP                     '\\'
@@ -66,8 +65,10 @@ b_bool hashCheckPassed = false;
 
 b_bool calculate_hashes(BitcolliderSubmission *submission,
                         FILE                  *source,
-                        unsigned char         *sha1, 
-                        unsigned char         *tigerTree,
+                        char                  *bitprint, 
+                        char                  *md5sum,
+						char                  *ed2kmd4sum,
+						char                  *ftuusum,
                         mp3_info              *mp3Info,
                         PluginMethods         *methods,
                         Attribute            **attrList);
@@ -77,18 +78,19 @@ b_bool generate_first_n_hex(BitcolliderSubmission *submission,
                             unsigned char *bitprint);
 b_bool get_bitprint_data(BitcolliderSubmission *submission,
                          const char            *fileName, 
-                         unsigned char         *sha1, 
-                         unsigned char         *tigerTree,
+                         char                  *bitprint, 
+                         char                  *md5sum,
+						 char                  *ed2kmd4sum,
+						 char                  *ftuusum,
                          unsigned char         *firstHex,
                          mp3_info              *mp3Info,
                          PluginMethods         *methods,
                          Attribute            **attrList);
 void   convert_to_multiple_submission(BitcolliderSubmission *submission);
-b_bool hash_sanity_check(void);
-b_bool check_tigertree_hash(const char *result, unsigned char *data, int len);
-b_bool check_sha1_hash(const char *result, unsigned char *data, int len);
 void   set_error(BitcolliderSubmission *sub, const char *newError);
 void   set_warning(BitcolliderSubmission *sub, const char *newError);
+b_bool check_md5_hash(void);
+char  *escape_form_value(char *form_value);
 /*------------------------------------------------------------------------- */
 
 Bitcollider *bitcollider_init(b_bool printDebugInfo)
@@ -98,26 +100,33 @@ Bitcollider *bitcollider_init(b_bool printDebugInfo)
     HKEY         hKey;
     DWORD        type;
 #endif
-    char         path[MAX_PATH], *ptr;
+    char         path[MAX_PATH], cacheFile[MAX_PATH], *ptr;
     int          total = 0;
     
+    cacheFile[0] = 0;
     bc = init_plugins();
 
 #ifndef WIN32
     /* Load the plugins from the build dir first if ./plugins exists */
     if (printDebugInfo)
-        printf("Loading plugins from ./plugins:\n"); 
+        fprintf(stderr, "Loading plugins from ./plugins:\n"); 
         
     total = load_plugins(bc, "./plugins", printDebugInfo);
 
     ptr = getenv("HOME");
     if (ptr)
     {
+        sprintf(cacheFile, "%s/.bitcollider/cache.db", ptr);
         sprintf(path, "%s/.bitcollider/plugins", ptr);
         if (printDebugInfo)
-            printf("Loading plugins from %s:\n", path); 
+            fprintf(stderr, "Loading plugins from %s:\n", path); 
         total += load_plugins(bc, path, printDebugInfo);
     }
+    else
+    {
+        if (printDebugInfo)
+            fprintf(stderr, "HOME env var not set. Cannot find home.\n"); 
+    }
 
     ptr = PREFIX"/lib/bitcollider/plugins";
 #else
@@ -136,17 +145,24 @@ Bitcollider *bitcollider_init(b_bool printDebugInfo)
         int   ret;
         DWORD size = MAX_PATH;
 
+        // Get the plugin dir, first
         ret = RegQueryValueEx(hKey, "PluginDir", NULL, &type, 
                               path, &size);
-        RegCloseKey(hKey);
 
         if (ret == ERROR_FILE_NOT_FOUND)
             set_plugin_dir(path);
         else
         if (ret != ERROR_SUCCESS)
+        {
+            RegCloseKey(hKey);
             return bc;
+        }
 
-        /*printf("Reg key set. Using path: '%s'\n", path); */
+        // Now try to get the cache file
+        size = MAX_PATH;
+        ret = RegQueryValueEx(hKey, "CacheFile", NULL, &type, 
+                              cacheFile, &size);
+        RegCloseKey(hKey);
     }
     ptr = path;
 #endif
@@ -154,17 +170,47 @@ Bitcollider *bitcollider_init(b_bool printDebugInfo)
     if (ptr)
     {
         if (printDebugInfo)
-            printf("Loading plugins from %s:\n", ptr); 
+            fprintf(stderr, "Loading plugins from %s:\n", ptr); 
         total += load_plugins(bc, ptr, printDebugInfo);
     }
     if (printDebugInfo)
-        printf("Loaded %d plugins total.\n\n", total);
+        fprintf(stderr, "Loaded %d plugins total.\n\n", total);
+
+#if USE_BDB
+    bc->maxCacheSize = DEFAULT_MAX_CACHE_SIZE;
+    if (cacheFile[0] != 0)
+    {
+        bc->cache = init_cache();
+        if (bc->cache)
+        {
+            if (!open_cache(bc->cache, cacheFile))
+            {
+                if (printDebugInfo)
+                    fprintf(stderr, "Failed to open cache file: %s. "
+                                    "Creating a new one.\n\n", cacheFile);
+                unlink(cacheFile);
+                if (!create_cache(bc->cache, cacheFile, bc->maxCacheSize))
+                {
+                    if (printDebugInfo)
+                        fprintf(stderr, "Failed to create cache file: %s.\n\n", 
+                                    cacheFile);
+                    close_cache(bc->cache);
+                    bc->cache = NULL;
+                }
+            }
+        }
+    }
+#endif
     
     return bc;
 }
 
 void bitcollider_shutdown(Bitcollider *bc)
 {
+#if USE_BDB
+    if (bc->cache)
+       close_cache(bc->cache);
+#endif
     unload_plugins(bc);
     shutdown_plugins(bc);
 }
@@ -180,13 +226,6 @@ BitcolliderSubmission *create_submission(Bitcollider *bc)
     memset(submission, 0, sizeof(BitcolliderSubmission));
     submission->bc = bc;
 
-    if (!hashCheckPassed && hash_sanity_check())
-    {
-        set_error(submission, ERROR_HASHCHECK);
-    }
-    else
-        hashCheckPassed = true;
-
     return submission;
 }
 
@@ -194,16 +233,22 @@ b_bool analyze_file(BitcolliderSubmission *submission,
                     const char            *fileName,
                     b_bool                 matchingExtsOnly)
 {
-    char                   shaDigest[SHA_HEXSIZE];
-    char                   tigerTreeDigest[TIGER_HEXSIZE];
+    char                   bitprint[BITPRINT_BASE32_LEN + 1];
     char                   firstNHex[FIRST_N_HEX_SIZE + 1];
     char                   temp[MAX_ATTR_STRING_LEN], *ext;
+    char                   md5[64];
+	char                   ed2kmd4[64];
+	char                   ftuu[64];
     const char            *baseFileName;
     mp3_info              *mp3Info = NULL;
     PluginMethods         *methods = NULL;
     b_bool                 mp3Check = false;
     Attribute             *attrList = NULL, *attr;
     const char            *err;
+#if USE_BDB
+    cache_entry            centry;
+    struct stat            fileInfo;
+#endif
 
     if (submission->bc->error)
     {
@@ -216,16 +261,16 @@ b_bool analyze_file(BitcolliderSubmission *submission,
         submission->bc->warning = NULL;
     }
 
-    if (!hashCheckPassed)
-    {
-        set_error(submission, ERROR_HASHCHECK);
-        return false;
-    }
-
     if (submission->fileName)
     {
-       free(submission->fileName);
-       submission->fileName = NULL;
+        free(submission->fileName);
+        submission->fileName = NULL;
+    }
+
+    if (!check_md5_hash())
+    {
+        set_error(submission, MD5_SANITY_CHECK_FAILED);
+        return false;
     }
 
     if (submission->bc->exitNow)
@@ -265,11 +310,44 @@ b_bool analyze_file(BitcolliderSubmission *submission,
     if (submission->bc->preview)
        return true;
 
+#if USE_BDB
+    /* Check to see if this bitprint has already been bitprinted */
+#ifdef WIN32
+    {
+       char *dummy;
+       if (GetFullPathName(fileName, MAX_PATH, centry.fileName, &dummy) == 0)
+           strcpy(centry.fileName, fileName);
+    }
+#else
+    if (realpath(fileName, centry.fileName) == NULL)
+        strcpy(centry.fileName, fileName);
+#endif
+
+    if (submission->bc->cache && 
+        get_cache_entry(submission->bc->cache, &centry))
+    {
+        if (_stat(submission->fileName, (struct _stat *)&fileInfo) == 0)
+        {
+            if (fileInfo.st_mtime == centry.lastModDate)
+            {
+                if (submission->bc->progressCallback && 
+                    !submission->bc->preview)
+                    submission->bc->progressCallback(0, submission->fileName, 
+                                   "found in cache, skipped.");
+                return true;
+            }
+        }
+        /* Its out of date or something else is wrong. Remove the entry
+           from the cache */
+        remove_cache_entry(submission->bc->cache, &centry);
+    }
+#endif
+
     if (mp3Check)
        mp3Info = malloc(sizeof(mp3_info));
 
-    if (!get_bitprint_data(submission, fileName, shaDigest, tigerTreeDigest, 
-                           firstNHex, mp3Info, methods, &attrList))
+    if (!get_bitprint_data(submission, fileName, bitprint, md5, ed2kmd4, ftuu, firstNHex, 
+                           mp3Info, methods, &attrList))
     {
        if (mp3Info)
           free(mp3Info);
@@ -292,13 +370,15 @@ b_bool analyze_file(BitcolliderSubmission *submission,
     if (submission->numBitprints == 1)
        convert_to_multiple_submission(submission);
 
-    sprintf(temp, "%s%s", shaDigest, tigerTreeDigest);
-    add_attribute(submission, "bitprint", temp);
+    add_attribute(submission, "bitprint", bitprint);
 
     sprintf(temp, "%lu", submission->fileSize);
     add_attribute(submission, "tag.file.length", temp);
     add_attribute(submission, "tag.file.first20", firstNHex);
     add_attribute(submission, "tag.filename.filename", baseFileName);
+    // add_attribute(submission, "tag.md5.md5", md5);
+    add_attribute(submission, "tag.ed2k.ed2khash", ed2kmd4);
+	add_attribute(submission, "tag.uuhash.uuhash", ftuu);
 
     /* Check to make sure that we carried out the mp3 check, and
        make sure that an audioSha was generated. If not, then the
@@ -312,10 +392,10 @@ b_bool analyze_file(BitcolliderSubmission *submission,
     else
     if (mp3Check)
     {
-       char audioShaDigest[SHA_HEXSIZE];
+       char audioShaDigest[SHA_BASE32SIZE + 1];
        ID3Info *info;
 
-       convert_to_hex(mp3Info->audioSha, SHA_DIGESTSIZE, audioShaDigest);
+       bitziEncodeBase32(mp3Info->audioSha, SHA_DIGESTSIZE, audioShaDigest);
        sprintf(temp, "%d", mp3Info->duration);
        add_attribute(submission, "tag.mp3.duration", temp);
        if (mp3Info->bitrate == 0)
@@ -389,6 +469,18 @@ b_bool analyze_file(BitcolliderSubmission *submission,
 
     submission->numBitprints++;
 
+#if USE_BDB
+    if (_stat(submission->fileName, (struct _stat *)&fileInfo) == 0)
+    {
+        centry.lastModDate = fileInfo.st_mtime;
+        if (submission->bc->cache && 
+            !add_cache_entry(submission->bc->cache, &centry, false))
+        {
+            printf("Failed to insert bitprint into cache.\n");        
+        }
+    }
+#endif
+
     return true;
 }
 
@@ -425,6 +517,26 @@ void set_exit(Bitcollider *bc, b_bool exitNow)
     bc->exitNow = exitNow;
 }
 
+void clear_bitprint_cache(Bitcollider *bc)
+{
+#if USE_BDB
+    if (bc->cache)
+    {
+        int maxCount;
+        char fileName[MAX_PATH];
+    
+        strcpy(fileName, bc->cache->fileName);
+        maxCount = get_max_count(bc->cache);
+        close_cache(bc->cache);
+        unlink(fileName);
+    
+        bc->cache = init_cache();
+        create_cache(bc->cache, fileName, maxCount);
+    }
+#endif
+}
+
+
 int get_num_bitprints(BitcolliderSubmission *sub)
 {
     return sub->numBitprints;
@@ -493,6 +605,19 @@ void convert_to_multiple_submission(BitcolliderSubmission *submission)
     }
 }
 
+void convert_to_hex(const unsigned char *buffer, 
+                    int size, 
+                    char *hexBuffer)
+{
+    int i;
+
+    for(i = 0; i < size; i++)
+    {
+        sprintf(hexBuffer + (i * sizeof(char) * 2), "%02X", buffer[i] & 0xFF);
+    }
+}
+
+
 void add_attribute(BitcolliderSubmission *submission, 
                    const char *key,
                    const char *value)
@@ -563,8 +688,10 @@ const char *get_attribute(BitcolliderSubmission *submission,
 
 b_bool get_bitprint_data(BitcolliderSubmission *submission,
                          const char            *fileName, 
-                         unsigned char         *sha1, 
-                         unsigned char         *tigerTree,
+                         char                  *bitprint, 
+                         char                  *md5sum,
+						 char                  *ed2kmd4,
+						 char                  *ftuu,
                          unsigned char         *firstHex,
                          mp3_info              *mp3Info,
                          PluginMethods         *methods,
@@ -584,7 +711,7 @@ b_bool get_bitprint_data(BitcolliderSubmission *submission,
     submission->fileSize = ftell(source);
     fseek(source, 0, SEEK_SET);
 
-    ret = calculate_hashes(submission, source, sha1, tigerTree,  
+    ret = calculate_hashes(submission, source, bitprint, md5sum, ed2kmd4, ftuu,
                            mp3Info, methods, attrList);
     if (ret)
         ret = generate_first_n_hex(submission, source, FIRST_N_HEX, firstHex);
@@ -595,27 +722,36 @@ b_bool get_bitprint_data(BitcolliderSubmission *submission,
 
 b_bool calculate_hashes(BitcolliderSubmission *submission,
                         FILE                  *source,
-                        unsigned char         *sha1, 
-                        unsigned char         *tigerTree,
+                        char                  *bitprint, 
+                        char                  *md5sum,
+						char                  *ed2kmd4sum,
+						char                  *ftuusum,
                         mp3_info              *mcontext,
                         PluginMethods         *methods,
                         Attribute            **attrList)     
 {
-    TT_CONTEXT      tcontext;
-    SHA_INFO        scontext;
-    unsigned char  *buffer;
-    int             bytes;
-    unsigned char   tigerHash[TIGERSIZE];
-    unsigned char   shaHash[SHA_DIGESTSIZE];
-    b_bool          ret = true;
-    Context         context = NULL;
+    BP_CONTEXT         bcontext;
+    struct MD5Context  md5context; 
+	ED2K_CTX           ed2kmd4context;
+	FTUU_CTX           ftuucontext;
+    unsigned char     *buffer, bitprintRaw[BITPRINT_RAW_LEN], md5Digest[16], ed2kDigest[16], ftuuDigest[20];
+    int                bytes;
+    b_bool             ret = true;
+    Context           *context = NULL;
+
+    if (bitziBitprintInit(&bcontext) == -1)
+    {
+        set_error(submission, ERROR_HASHCHECK);
+        return false;
+    }
 
-    tt_init(&tcontext);
-    sha_init(&scontext);
     if (mcontext)
        mp3_init(mcontext);
     if (methods && methods->mem_analyze_init)
        context = methods->mem_analyze_init();
+    // MD5Init(&md5context);
+	ED2KInit(&ed2kmd4context);
+	FTUUInit(&ftuucontext);
 
     buffer = (unsigned char*)malloc(BUFFER_LEN);
     if (buffer == NULL)
@@ -641,18 +777,21 @@ b_bool calculate_hashes(BitcolliderSubmission *submission,
            break;
         }
 
-        tt_update(&tcontext, buffer, bytes);
-        sha_update(&scontext, buffer, bytes);
+        bitziBitprintUpdate(&bcontext, buffer, bytes);
         if (mcontext)
            mp3_update(mcontext, buffer, bytes);
         if (methods && methods->mem_analyze_update)
            methods->mem_analyze_update(context, buffer, bytes);
-
+        // MD5Update(&md5context, buffer, bytes);
+		ED2KUpdate(&ed2kmd4context, buffer, bytes);
+		FTUUUpdate(&ftuucontext, buffer, bytes);
+        
         if (submission->bc->progressCallback && !submission->bc->preview)
         {
            int          percentComplete;
 
-           percentComplete = ((ftell(source) * 100) / submission->fileSize);
+           percentComplete = (int)(((word64)ftell(source) * (word64)100) / 
+                                    (word64)submission->fileSize);
            if (percentComplete != submission->percentComplete)
            {
                submission->bc->progressCallback(percentComplete, NULL, NULL);
@@ -664,15 +803,33 @@ b_bool calculate_hashes(BitcolliderSubmission *submission,
 
     free(buffer);
 
-    tt_digest(&tcontext, tigerHash);
-    sha_final(shaHash, &scontext);
+    bitziBitprintFinal(&bcontext, bitprintRaw);
+    bitziBitprintToBase32(bitprintRaw, bitprint);
+
     if (mcontext)
         mp3_final(mcontext);
     if (methods && methods->mem_analyze_final)
         *attrList = methods->mem_analyze_final(context);
 
-    convert_to_hex(tigerHash, TIGERSIZE, tigerTree);
-    convert_to_hex(shaHash, SHA_DIGESTSIZE, sha1);
+    // MD5Final(md5Digest, &md5context);
+	ED2KFinal(ed2kDigest, &ed2kmd4context);
+	FTUUFinal(ftuuDigest, &ftuucontext);
+
+    bitziEncodeBase32(md5Digest, 16, md5sum);
+    sprintf(ed2kmd4sum,"%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
+		    ed2kDigest[0],ed2kDigest[1],ed2kDigest[2],ed2kDigest[3],
+			ed2kDigest[4],ed2kDigest[5],ed2kDigest[6],ed2kDigest[7],
+			ed2kDigest[8],ed2kDigest[9],ed2kDigest[10],ed2kDigest[11],
+			ed2kDigest[12],ed2kDigest[13],ed2kDigest[14],ed2kDigest[15]);
+    /**
+	sprintf(ftuusum,"%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
+		    ftuuDigest[0],ftuuDigest[1],ftuuDigest[2],ftuuDigest[3],
+			ftuuDigest[4],ftuuDigest[5],ftuuDigest[6],ftuuDigest[7],
+			ftuuDigest[8],ftuuDigest[9],ftuuDigest[10],ftuuDigest[11],
+			ftuuDigest[12],ftuuDigest[13],ftuuDigest[14],ftuuDigest[15],
+			ftuuDigest[16],ftuuDigest[17],ftuuDigest[18],ftuuDigest[19]);
+	**/
+	bitziEncodeBase64(ftuuDigest,20,ftuusum);
 
     return ret;
 }
@@ -707,23 +864,13 @@ b_bool generate_first_n_hex(BitcolliderSubmission *submission,
     return true;
 }
 
-void convert_to_hex(const unsigned char *buffer, int size, char *hexBuffer)
-{
-    int i;
-
-    for(i = 0; i < size; i++)
-    {
-        sprintf(hexBuffer + (i * sizeof(char) * 2), "%02X", buffer[i] & 0xFF);
-    }
-}
-
 b_bool submit_submission(BitcolliderSubmission *submission, 
                          const char *url, 
                          BrowserEnum browser)
 {
     FILE *output;
     int   i, last = -1;
-    char  tempFile[MAX_PATH];
+    char  tempFile[MAX_PATH], *escaped;
     b_bool  ret;
 
     if (submission->numBitprints == 0)
@@ -733,7 +880,7 @@ b_bool submit_submission(BitcolliderSubmission *submission,
     }
 
 #ifdef _WIN32
-	 GetTempPath(MAX_PATH, tempFile);
+         GetTempPath(MAX_PATH, tempFile);
     strcat(tempFile, "bitprint.htm");
 #else
     strcpy(tempFile, "/tmp/bitprint.html");
@@ -789,10 +936,12 @@ b_bool submit_submission(BitcolliderSubmission *submission,
         }
         fprintf(output, "%s=<INPUT TYPE=\"hidden\" ",
                 submission->attrList[i]->key);
+        escaped = escape_form_value(submission->attrList[i]->value);
         fprintf(output, "NAME=\"%s\" VALUE=\"%s\">%s\n",
                 submission->attrList[i]->key,
-                submission->attrList[i]->value,
+                escaped,
                 submission->attrList[i]->value);
+        free(escaped);
     }
     fprintf(output, "\n<INPUT TYPE=\"submit\" NAME=\"Submit\" VALUE=\"Submit\">\n");
     fprintf(output, "</PRE>\n</FORM>\n</BODY>\n</HTML>\n");
@@ -821,11 +970,182 @@ b_bool submit_submission(BitcolliderSubmission *submission,
     return ret;
 }
 
+char *escape_form_value(char *form_value)
+{
+    int i, form_value_length, extra_length;
+    char *escaped_value, *ptr;
+
+    form_value_length = strlen(form_value);
+    for (i = 0, extra_length = 0; i < form_value_length; ++i)
+    {
+        switch(form_value[i])
+        {
+            case '"':
+                extra_length += 5;
+                break;
+            case '&':
+                extra_length += 4;
+                break;
+            case '<':
+            case '>':
+                extra_length += 3;
+                break;
+        }
+    }
+
+    if (extra_length == 0)
+    {
+        // This is necessary since the caller must free the memory.
+        return strdup(form_value);
+    }
+
+    escaped_value = malloc(form_value_length + extra_length + 1);
+    for (i = 0, ptr = escaped_value; i < form_value_length; ++i)
+    {
+        switch(form_value[i])
+        {
+            case '"':
+                strcpy(ptr, "&quot;");
+                ptr += 6;
+                break;
+            case '&':
+                strcpy(ptr, "&amp;");
+                ptr += 5;
+                break;
+            case '<':
+                strcpy(ptr, "&lt;");
+                ptr += 4;
+                break;
+            case '>':
+                strcpy(ptr, "&gt;");
+                ptr += 4;
+                break;
+            default:
+                *(ptr++) = form_value[i];
+        }
+    }
+    *ptr = 0;
+
+    return escaped_value;
+}
+
+BitcolliderSubmission *read_submission_from_file(Bitcollider *bc, char *fileName)
+{
+    BitcolliderSubmission *submission;
+    char                   buf[BUFFER_LEN], last[BUFFER_LEN], temp[BUFFER_LEN], err[255];
+    char                  *c, *t;
+    int                    line, empty = 1;
+    FILE                  *infile;
+
+    if (strcmp(fileName, "-"))
+       infile = stdin;
+    else
+       infile = fopen(fileName, "rb");
+
+    submission = (BitcolliderSubmission *)malloc(sizeof(BitcolliderSubmission));
+    if (submission == NULL)
+    {
+       fclose(infile);
+       return NULL;
+    }
+    memset(submission, 0, sizeof(BitcolliderSubmission));
+    submission->bc = bc;
+
+    if (infile == NULL)
+    {
+       sprintf(err, "Can't open tag file: %s", strerror(errno));
+       set_error(submission, err);
+       return submission;
+    }
+
+    last[0] = 0;
+
+    for( line = 1; fgets( buf, BUFFER_LEN, infile ) != NULL; ++line )
+    {
+       if( ! ( ( t = strchr( buf, '\r' ) ) || ( t = strchr( buf, '\n' ) ) ) )
+       {
+           if( strlen( buf ) == BUFFER_LEN - 1 )
+           {
+              sprintf(err, "Line %d exceeds length limit", line );
+              set_error(submission, err);
+              fclose(infile);
+              submission->numBitprints = 0;
+              return submission;
+           }
+           else
+           {
+              sprintf(err, "Line %d is truncated", line );
+              set_error(submission, err);
+              fclose(infile);
+              submission->numBitprints = 0;
+              return submission;
+           }
+       }
+       *t = 0;
+
+       for( c = buf; *c == ' ' && *c == '\t'; ++c )
+           ; /* <-- Empty for loop */
+
+       if( ! *c || *c == '#' ) 
+           continue;
+
+       if( ! ( t = strchr( c, '=' ) ) )
+       {
+           sprintf(err, "Line %d does not appear to contain a tag", line );
+           set_error(submission, err);
+           fclose(infile);
+           submission->numBitprints = 0;
+           return submission;
+       }
+       *t = 0;
+       if( ! strncmp( c, "head.", 5 ) ) 
+           continue;
+       if( isdigit( *c ) )
+       {
+           if( ! ( t = strchr( c, '.' ) ) )
+           {
+              sprintf(err, "Line %d does not appear to contain a tag", line);
+              set_error(submission, err);
+              fclose(infile);
+              submission->numBitprints = 0;
+              return submission;
+           }
+           *t = 0;
+
+           strcpy( last, c );
+           c = t + 1;
+       }
+       if (empty)
+       {
+           empty = 0;
+           get_agent_string(temp);
+           add_attribute(submission, "head.agent", temp);
+ 
+           sprintf(temp, "S%s", BC_SUBMITSPECVER);
+           add_attribute(submission, "head.version", temp);
+       }
+
+       if(strncmp(c, "bitprint", 8) == 0)
+       {
+           if (submission->numBitprints == 1)
+              convert_to_multiple_submission(submission);
+
+           submission->numBitprints++;
+       }
+       submission->numBitprints--;
+       add_attribute( submission, c, c + strlen( c ) + 1 );
+       submission->numBitprints++;
+    }
+
+    fclose(infile);
+
+    return submission;
+}
+
 void print_submission(BitcolliderSubmission *submission)
 {
     int i;
 
-    printf("\n");
     for(i = 0; i < submission->numItems; i++)
     {
         printf("%s=%s\n", 
@@ -834,70 +1154,6 @@ void print_submission(BitcolliderSubmission *submission)
     }
 }
 
-b_bool hash_sanity_check(void)
-{
-    b_bool         ret;
-    unsigned char *data; 
-
-
-    ret = check_tigertree_hash(EMPTY_TIGER, "", 0);
-    ret += check_sha1_hash(EMPTY_SHA, "", 0);
-    ret += check_tigertree_hash(ONE_TIGER, "1", 1);
-    ret += check_sha1_hash(ONE_SHA, "1", 1);
-
-    data = malloc(ONEK_SIZE);
-    memset(data, 'a', ONEK_SIZE);
-    ret += check_tigertree_hash(ONEK_TIGER, data, ONEK_SIZE);
-    ret += check_sha1_hash(ONEK_SHA, data, ONEK_SIZE);
-    free(data);
-
-    return ret;
-}
-
-/* NOTE: This function returns true if it failed the check! */
-b_bool check_tigertree_hash(const char *result, unsigned char *data, int len)
-{
-    TT_CONTEXT      tcontext;
-    unsigned char   tigerTreeHash[TIGERSIZE];
-    char            tigerTreeDigest[TIGER_HEXSIZE];
-
-    tt_init(&tcontext);
-    tt_update(&tcontext, data, len);
-    tt_digest(&tcontext, tigerTreeHash);
-    convert_to_hex(tigerTreeHash, TIGERSIZE, tigerTreeDigest);
-
-    if (strcmp(tigerTreeDigest, result))
-    {
-        fprintf(stderr, "        tigertree: '%s' [%d]\n", 
-                tigerTreeDigest, len);
-        fprintf(stderr, "correct tigertree: '%s' [%d]\n", 
-                result, len);
-        return true;
-    }
-    return false;
-}
-
-/* NOTE: This function returns true if it failed the check! */
-b_bool check_sha1_hash(const char *result, unsigned char *data, int len)
-{
-    SHA_INFO        scontext;
-    unsigned char   shaHash[SHA_DIGESTSIZE];
-    char            shaDigest[SHA_HEXSIZE];
-
-    sha_init(&scontext);
-    sha_update(&scontext, data, len);
-    sha_final(shaHash, &scontext);
-    convert_to_hex(shaHash, SHA_DIGESTSIZE, shaDigest);
-
-    if (strcmp(shaDigest, result))
-    {
-        fprintf(stderr, "              sha: '%s' [%d]\n", shaDigest, len);
-        fprintf(stderr, "      correct sha: '%s' [%d]\n", result, len);
-        return true;
-    }
-    return false;
-}
-
 void get_agent_string(char *agentString)
 {
     sprintf(agentString, "%s/%s (%s)", BC_AGENTNAME, BC_VERSION, 
@@ -928,3 +1184,28 @@ void getLongPathName(const char *shortName, int len, char *longName)
     longName[len - 1] = 0;
 }
 #endif
+
+b_bool check_md5_hash(void)
+{
+    struct MD5Context  md5context; 
+    unsigned char      md5Digest[16];
+    char               md5Hash[33];
+
+    MD5Init(&md5context);
+    MD5Final(md5Digest, &md5context);
+    bitziEncodeBase32(md5Digest, 16, md5Hash);
+
+    if (strcmp(MD5_SANITY_CHECK_EMPTY, md5Hash))
+        return false;
+
+    MD5Init(&md5context);
+    MD5Update(&md5context, "01234", 5);
+    MD5Final(md5Digest, &md5context);
+    bitziEncodeBase32(md5Digest, 16, md5Hash);
+
+    if (strcmp(MD5_SANITY_CHECK_01234, md5Hash))
+        return false;
+
+    return true;
+}
+
